/* Decimal floating point support for GDB.

   Copyright 2006 Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include "defs.h"
#include "dfp.h"
#include <ctype.h>

/* The order of the following headers is important for making sure
   decNumber structure is large enough to hold decimal128 digits.  */

#include "decimal128.h"
#include "decimal64.h"
#include "decimal32.h"
#include "decNumber.h"

/* In gdb, we are using an array of gdb_byte to represent decimal values. The
   byte order of our representation might be different than that of underlying
   architecture.  This routine does the conversion if it is necessary.  */
static void
exchange_dfp (const gdb_byte *valaddr, int len, gdb_byte *dec_val)
{
  int index;

  if (gdbarch_byte_order (current_gdbarch) == 1)
    for (index = 0; index < len; index++)
      dec_val[index] = valaddr[len - index - 1];
  else
    for (index = 0; index < len; index++)
      dec_val[index] = valaddr[index];

  return;
}

/* Convert deciaml type to its string representation.  LEN is the length
   of the decimal type, 4 bytes for decimal32, 8 bytes for decimal64 and
   16 bytes for decimal128.  */
void
decimal_to_string (const uint8_t *decbytes, int len, char *s)
{
  uint8_t *dec = (uint8_t *)malloc (len);

  exchange_dfp (decbytes, len, dec);
  switch (len)
    {
      case 4:
        decimal32ToString ((decimal32 *) dec, s);
        return;
      case 8:
        decimal64ToString ((decimal64 *) dec, s);
        return;
      case 16:
        decimal128ToString ((decimal128 *) dec, s);
        return;
      default:
	return;
    }

  free (dec);
}

/* Convert the string form of a decimal value to its decimal representation.
   LEN is the length of the decimal type, 4 bytes for decimal32, 8 bytes for
   decimal64 and 16 bytes for decimal128.  */
int
decimal_from_string (uint8_t *decbytes, int len, char *string)
{
  decNumber dn;
  decContext set;
  int index;
  uint8_t *dec = (uint8_t *)malloc (len);

  switch (len)
    {
      case 4:
	decContextDefault (&set, DEC_INIT_DECIMAL32);
	break;
      case 8:
	decContextDefault (&set, DEC_INIT_DECIMAL64);
	break;
      case 16:
	decContextDefault (&set, DEC_INIT_DECIMAL128);
	break;
    }

  set.traps = 0;

  decNumberFromString (&dn, string, &set);
  switch (len)
    {
      case 4:
	decimal32FromNumber ((decimal32 *) dec, &dn, &set);
	break;
      case 8:
	decimal64FromNumber ((decimal64 *) dec, &dn, &set);
	break;
      case 16:
	decimal128FromNumber ((decimal128 *) dec, &dn, &set);
	break;
    }

  exchange_dfp (dec, len, decbytes);
  free (dec);
  return 1;
}
